
"""
List of elements

Data processed from wikipedia's periodic table template page (fall 2014)
"""

elements = [[
    {'Z': 1, 'name': "hydrogen", 'id': "H", 'phase': "Gas  ",
        'category': "Diatomic nonmetal", 'occurrence': "Primordial"},
    (17, ""),
    {'Z': 2, 'name': "helium", 'id': "He", 'phase': "Gas  ",
        'category': "Noble gas", 'occurrence': "Primordial"},

], [
    {'Z': 3, 'name': "lithium", 'id': "Li", 'phase': "Solid",
        'category': "Alkali metal", 'occurrence': "Primordial"},
    {'Z': 4, 'name': "beryllium", 'id': "Be", 'phase': "Solid",
        'category': "Alkaline earth metal", 'occurrence': "Primordial"},
    (11, ""),
    {'Z': 5, 'name': "boron", 'id': "B", 'phase': "Solid",
        'category': "Metalloid", 'occurrence': "Primordial"},
    {'Z': 6, 'name': "carbon", 'id': "C", 'phase': "Solid",
        'category': "Polyatomic nonmetal", 'occurrence': "Primordial"},
    {'Z': 7, 'name': "nitrogen", 'id': "N", 'phase': "Gas  ",
        'category': "Diatomic nonmetal", 'occurrence': "Primordial"},
    {'Z': 8, 'name': "oxygen", 'id': "O", 'phase': "Gas  ",
        'category': "Diatomic nonmetal", 'occurrence': "Primordial"},
    {'Z': 9, 'name': "fluorine", 'id': "F", 'phase': "Gas  ",
        'category': "Diatomic nonmetal", 'occurrence': "Primordial"},
    {'Z': 10, 'name': "neon", 'id': "Ne", 'phase': "Gas  ",
        'category': "Noble gas", 'occurrence': "Primordial"},

], [
    {'Z': 11, 'name': "sodium", 'id': "Na", 'phase': "Solid",
        'category': "Alkali metal", 'occurrence': "Primordial"},
    {'Z': 12, 'name': "magnesium", 'id': "Mg", 'phase': "Solid",
        'category': "Alkaline earth metal", 'occurrence': "Primordial"},
    (11, ""),
    {'Z': 13, 'name': "aluminium", 'id': "Al", 'phase': "Solid",
        'category': "post-transition metal", 'occurrence': "Primordial"},
    {'Z': 14, 'name': "silicon", 'id': "Si", 'phase': "Solid",
        'category': "Metalloid", 'occurrence': "Primordial"},
    {'Z': 15, 'name': "phosphorus", 'id': "P", 'phase': "Solid",
        'category': "Polyatomic nonmetal", 'occurrence': "Primordial"},
    {'Z': 16, 'name': "sulfur", 'id': "S", 'phase': "Solid",
        'category': "Polyatomic nonmetal", 'occurrence': "Primordial"},
    {'Z': 17, 'name': "chlorine", 'id': "Cl", 'phase': "Gas  ",
        'category': "Diatomic nonmetal", 'occurrence': "Primordial"},
    {'Z': 18, 'name': "argon", 'id': "Ar", 'phase': "Gas  ",
        'category': "Noble gas", 'occurrence': "Primordial"},

], [
    {'Z': 19, 'name': "potassium", 'id': "K", 'phase': "Solid",
        'category': "Alkali metal", 'occurrence': "Primordial"},
    {'Z': 20, 'name': "calcium", 'id': "Ca", 'phase': "Solid",
        'category': "Alkaline earth metal", 'occurrence': "Primordial"},
    (1, ""),
    {'Z': 21, 'name': "scandium", 'id': "Sc", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 22, 'name': "titanium", 'id': "Ti", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 23, 'name': "vanadium", 'id': "V", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 24, 'name': "chromium", 'id': "Cr", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 25, 'name': "manganese", 'id': "Mn", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 26, 'name': "iron", 'id': "Fe", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 27, 'name': "cobalt", 'id': "Co", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 28, 'name': "nickel", 'id': "Ni", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 29, 'name': "copper", 'id': "Cu", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 30, 'name': "zinc", 'id': "Zn", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 31, 'name': "gallium", 'id': "Ga", 'phase': "Solid",
        'category': "post-transition metal", 'occurrence': "Primordial"},
    {'Z': 32, 'name': "germanium", 'id': "Ge", 'phase': "Solid",
        'category': "Metalloid", 'occurrence': "Primordial"},
    {'Z': 33, 'name': "arsenic", 'id': "As", 'phase': "Solid",
        'category': "Metalloid", 'occurrence': "Primordial"},
    {'Z': 34, 'name': "selenium", 'id': "Se", 'phase': "Solid",
        'category': "Polyatomic nonmetal", 'occurrence': "Primordial"},
    {'Z': 35, 'name': "bromine", 'id': "Br", 'phase': "Liquid",
        'category': "Diatomic nonmetal", 'occurrence': "Primordial"},
    {'Z': 36, 'name': "krypton", 'id': "Kr", 'phase': "Gas  ",
        'category': "Noble gas", 'occurrence': "Primordial"},

], [
    {'Z': 37, 'name': "rubidium", 'id': "Rb", 'phase': "Solid",
        'category': "Alkali metal", 'occurrence': "Primordial"},
    {'Z': 38, 'name': "strontium", 'id': "Sr", 'phase': "Solid",
        'category': "Alkaline earth metal", 'occurrence': "Primordial"},
    (1, ""),
    {'Z': 39, 'name': "yttrium", 'id': "Y", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 40, 'name': "zirconium", 'id': "Zr", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 41, 'name': "niobium", 'id': "Nb", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 42, 'name': "molybdenum", 'id': "Mo", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 43, 'name': "technetium", 'id': "Tc", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "From decay"},
    {'Z': 44, 'name': "ruthenium", 'id': "Ru", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 45, 'name': "rhodium", 'id': "Rh", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 46, 'name': "palladium", 'id': "Pd", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 47, 'name': "silver", 'id': "Ag", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 48, 'name': "cadmium", 'id': "Cd", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 49, 'name': "indium", 'id': "In", 'phase': "Solid",
        'category': "post-transition metal", 'occurrence': "Primordial"},
    {'Z': 50, 'name': "tin", 'id': "Sn", 'phase': "Solid",
        'category': "post-transition metal", 'occurrence': "Primordial"},
    {'Z': 51, 'name': "antimony", 'id': "Sb", 'phase': "Solid",
        'category': "Metalloid", 'occurrence': "Primordial"},
    {'Z': 52, 'name': "tellurium", 'id': "Te", 'phase': "Solid",
        'category': "Metalloid", 'occurrence': "Primordial"},
    {'Z': 53, 'name': "iodine", 'id': "I", 'phase': "Solid",
        'category': "Diatomic nonmetal", 'occurrence': "Primordial"},
    {'Z': 54, 'name': "xenon", 'id': "Xe", 'phase': "Gas  ",
        'category': "Noble gas", 'occurrence': "Primordial"},

], [
    {'Z': 55, 'name': "caesium", 'id': "Cs", 'phase': "Solid",
        'category': "Alkali metal", 'occurrence': "Primordial"},
    {'Z': 56, 'name': "barium", 'id': "Ba", 'phase': "Solid",
        'category': "Alkaline earth metal", 'occurrence': "Primordial"},
    (1, "*"),
    {'Z': 71, 'name': "lutetium", 'id': "Lu", 'phase': "Solid",
        'category': "Lanthanide", 'occurrence': "Primordial"},
    {'Z': 72, 'name': "hafnium", 'id': "Hf", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 73, 'name': "tantalum", 'id': "Ta", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 74, 'name': "tungsten", 'id': "W", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 75, 'name': "rhenium", 'id': "Re", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 76, 'name': "osmium", 'id': "Os", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 77, 'name': "iridium", 'id': "Ir", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 78, 'name': "platinum", 'id': "Pt", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 79, 'name': "gold", 'id': "Au", 'phase': "Solid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 80, 'name': "mercury", 'id': "Hg", 'phase': "Liquid",
        'category': "Transition metal", 'occurrence': "Primordial"},
    {'Z': 81, 'name': "thallium", 'id': "Tl", 'phase': "Solid",
        'category': "post-transition metal", 'occurrence': "Primordial"},
    {'Z': 82, 'name': "lead", 'id': "Pb", 'phase': "Solid",
        'category': "post-transition metal", 'occurrence': "Primordial"},
    {'Z': 83, 'name': "bismuth", 'id': "Bi", 'phase': "Solid",
        'category': "post-transition metal", 'occurrence': "Primordial"},
    {'Z': 84, 'name': "polonium", 'id': "Po", 'phase': "Solid",
        'category': "post-transition metal", 'occurrence': "From decay"},
    {'Z': 85, 'name': "astatine", 'id': "At", 'phase': "Solid",
        'category': "Metalloid", 'occurrence': "From decay"},
    {'Z': 86, 'name': "radon", 'id': "Rn", 'phase': "Gas  ",
        'category': "Noble gas", 'occurrence': "From decay"},

], [
    {'Z': 87, 'name': "francium", 'id': "Fr", 'phase': "Solid",
        'category': "Alkali metal", 'occurrence': "From decay"},
    {'Z': 88, 'name': "radium", 'id': "Ra", 'phase': "Solid",
        'category': "Alkaline earth metal", 'occurrence': "From decay"},
    (1, "**"),
    {'Z': 103, 'name': "lawrencium", 'id': "Lr", 'phase': "Unknown phase",
        'category': "Actinide", 'occurrence': "Synthetic"},
    {'Z': 104, 'name': "rutherfordium", 'id': "Rf", 'phase': "Unknown phase",
        'category': "Transition metal", 'occurrence': "Synthetic"},
    {'Z': 105, 'name': "dubnium", 'id': "Db", 'phase': "Unknown phase",
        'category': "Transition metal", 'occurrence': "Synthetic"},
    {'Z': 106, 'name': "seaborgium", 'id': "Sg", 'phase': "Unknown phase",
        'category': "Transition metal", 'occurrence': "Synthetic"},
    {'Z': 107, 'name': "bohrium", 'id': "Bh", 'phase': "Unknown phase",
        'category': "Transition metal", 'occurrence': "Synthetic"},
    {'Z': 108, 'name': "hassium", 'id': "Hs", 'phase': "Unknown phase",
        'category': "Transition metal", 'occurrence': "Synthetic"},
    {'Z': 109, 'name': "meitnerium", 'id': "Mt", 'phase': "Unknown phase",
        'category': "Unknown chemical properties", 'occurrence': "Synthetic"},
    {'Z': 110, 'name': "darmstadtium", 'id': "Ds", 'phase': "Unknown phase",
        'category': "Unknown chemical properties", 'occurrence': "Synthetic"},
    {'Z': 111, 'name': "roentgenium", 'id': "Rg", 'phase': "Unknown phase",
        'category': "Unknown chemical properties", 'occurrence': "Synthetic"},
    {'Z': 112, 'name': "copernicium", 'id': "Cn", 'phase': "Unknown phase",
        'category': "Transition metal", 'occurrence': "Synthetic"},
    {'Z': 113, 'name': "ununtrium", 'id': "Uut", 'phase': "Unknown phase",
        'category': "Unknown chemical properties", 'occurrence': "Synthetic"},
    {'Z': 114, 'name': "flerovium", 'id': "Fl", 'phase': "Unknown phase",
        'category': "Post-transition metal", 'occurrence': "Synthetic"},
    {'Z': 115, 'name': "ununpentium", 'id': "Uup", 'phase': "Unknown phase",
        'category': "Unknown chemical properties", 'occurrence': "Synthetic"},
    {'Z': 116, 'name': "livermorium", 'id': "Lv", 'phase': "Unknown phase",
        'category': "Unknown chemical properties", 'occurrence': "Synthetic"},
    {'Z': 117, 'name': "ununseptium", 'id': "Uus", 'phase': "Unknown phase",
        'category': "Unknown chemical properties", 'occurrence': "Synthetic"},
    {'Z': 118, 'name': "ununoctium", 'id': "Uuo", 'phase': "Unknown phase",
        'category': "Unknown chemical properties", 'occurrence': "Synthetic"},

], [
    (3, "*"),
    {'Z': 57, 'name': "lanthanum", 'id': "La", 'phase': "Solid",
        'category': "Lanthanide", 'occurrence': "Primordial"},
    {'Z': 58, 'name': "cerium", 'id': "Ce", 'phase': "Solid",
        'category': "Lanthanide", 'occurrence': "Primordial"},
    {'Z': 59, 'name': "praseodymium", 'id': "Pr", 'phase': "Solid",
        'category': "Lanthanide", 'occurrence': "Primordial"},
    {'Z': 60, 'name': "neodymium", 'id': "Nd", 'phase': "Solid",
        'category': "Lanthanide", 'occurrence': "Primordial"},
    {'Z': 61, 'name': "promethium", 'id': "Pm", 'phase': "Solid",
        'category': "Lanthanide", 'occurrence': "From decay"},
    {'Z': 62, 'name': "samarium", 'id': "Sm", 'phase': "Solid",
        'category': "Lanthanide", 'occurrence': "Primordial"},
    {'Z': 63, 'name': "europium", 'id': "Eu", 'phase': "Solid",
        'category': "Lanthanide", 'occurrence': "Primordial"},
    {'Z': 64, 'name': "gadolinium", 'id': "Gd", 'phase': "Solid",
        'category': "Lanthanide", 'occurrence': "Primordial"},
    {'Z': 65, 'name': "terbium", 'id': "Tb", 'phase': "Solid",
        'category': "Lanthanide", 'occurrence': "Primordial"},
    {'Z': 66, 'name': "dysprosium", 'id': "Dy", 'phase': "Solid",
        'category': "Lanthanide", 'occurrence': "Primordial"},
    {'Z': 67, 'name': "holmium", 'id': "Ho", 'phase': "Solid",
        'category': "Lanthanide", 'occurrence': "Primordial"},
    {'Z': 68, 'name': "erbium", 'id': "Er", 'phase': "Solid",
        'category': "Lanthanide", 'occurrence': "Primordial"},
    {'Z': 69, 'name': "thulium", 'id': "Tm", 'phase': "Solid",
        'category': "Lanthanide", 'occurrence': "Primordial"},
    {'Z': 70, 'name': "ytterbium", 'id': "Yb", 'phase': "Solid",
        'category': "Lanthanide", 'occurrence': "Primordial"},

], [
    (3, "**"),
    {'Z': 89, 'name': "actinium", 'id': "Ac", 'phase': "Solid",
        'category': "Actinide", 'occurrence': "From decay"},
    {'Z': 90, 'name': "thorium", 'id': "Th", 'phase': "Solid",
        'category': "Actinide", 'occurrence': "Primordial"},
    {'Z': 91, 'name': "protactinium", 'id': "Pa", 'phase': "Solid",
        'category': "Actinide", 'occurrence': "From decay"},
    {'Z': 92, 'name': "uranium", 'id': "U", 'phase': "Solid",
        'category': "Actinide", 'occurrence': "Primordial"},
    {'Z': 93, 'name': "neptunium", 'id': "Np", 'phase': "Solid",
        'category': "Actinide", 'occurrence': "From decay"},
    {'Z': 94, 'name': "plutonium", 'id': "Pu", 'phase': "Solid",
        'category': "Actinide", 'occurrence': "Primordial"},
    {'Z': 95, 'name': "americium", 'id': "Am", 'phase': "Solid",
        'category': "Actinide", 'occurrence': "From decay"},
    {'Z': 96, 'name': "curium", 'id': "Cm", 'phase': "Solid",
        'category': "Actinide", 'occurrence': "From decay"},
    {'Z': 97, 'name': "berkelium", 'id': "Bk", 'phase': "Solid",
        'category': "Actinide", 'occurrence': "From decay"},
    {'Z': 98, 'name': "californium", 'id': "Cf", 'phase': "Solid",
        'category': "Actinide", 'occurrence': "From decay"},
    {'Z': 99, 'name': "einsteinium", 'id': "Es", 'phase': "Solid",
        'category': "Actinide", 'occurrence': "Synthetic"},
    {'Z': 100, 'name': "fermium", 'id': "Fm", 'phase': "Unknown phase",
        'category': "Actinide", 'occurrence': "Synthetic"},
    {'Z': 101, 'name': "mendelevium", 'id': "Md", 'phase': "Unknown phase",
        'category': "Actinide", 'occurrence': "Synthetic"},
    {'Z': 102, 'name': "nobelium", 'id': "No", 'phase': "Unknown phase",
        'category': "Actinide", 'occurrence': "Synthetic"}
]]
